<?php

declare(strict_types = 1);

namespace Amazingcat\BraintreeVault\Controller;

use Braintree\Gateway as BraintreeGateway;
use Magento\Braintree\Gateway\Config\Config as BraintreeConfig;
use Magento\Customer\Model\Customer;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Request\Http;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\Stdlib\DateTime\DateTime;
use Magento\Framework\Data\Form\FormKey\Validator;
use Magento\Vault\Api\PaymentTokenRepositoryInterface;
use Magento\Vault\Model\PaymentTokenManagement;

/**
 * Class CardAction
 * @package Amazingcat\BraintreeVault\Controller
 */
abstract class CardAction extends Action
{
    const OK = 0;

    const WRONG_REQUEST = 1;

    const WRONG_TOKEN = 2;

    const ACTION_EXCEPTION = 3;

    const METHOD_CODE = 'braintree';

    const CARD_TYPES = [
        'American Express' => 'AE',
        'Visa' => 'VI',
        'MasterCard' => 'MC',
        'Discover' => 'DI',
        'JBC' => 'JBC',
        'China Union Pay' => 'CUP',
        'Maestro' => 'MI'
    ];

    /**
     * @var array
     */
    protected $errorsMap = [];

    /**
     * @var SerializerInterface
     */
    protected $serializer;

    /**
     * @var DateTime
     */
    protected $dateTime;

    /**
     * @var PaymentTokenRepositoryInterface
     */
    protected $tokenRepository;

    /**
     * @var PaymentTokenManagement
     */
    protected $paymentTokenManagement;

    /**
     * @var Customer
     */
    protected $customer;

    /**
     * @var BraintreeGateway|null
     */
    protected $braintreeGateway = null;

    /**
     * @var Validator
     */
    private $fkValidator;

    /**
     * @var BraintreeConfig
     */
    private $braintreeConfig;

    /**
     * @var Http
     */
    private $request;

    /**
     * @var String
     */
    private $payloadNonce;

    /**
     * @param Context $context
     * @param Session $customerSession
     * @param SerializerInterface $serializer
     * @param DateTime $dateTime
     * @param Validator $fkValidator
     * @param PaymentTokenRepositoryInterface $tokenRepository
     * @param PaymentTokenManagement $paymentTokenManagement
     * @param BraintreeConfig $braintreeConfig
     */
    public function __construct(
        Context $context,
        Session $customerSession,
        SerializerInterface $serializer,
        DateTime $dateTime,
        Validator $fkValidator,
        PaymentTokenRepositoryInterface $tokenRepository,
        PaymentTokenManagement $paymentTokenManagement,
        BraintreeConfig $braintreeConfig
    ) {
        $this->serializer = $serializer;
        $this->dateTime = $dateTime;
        $this->fkValidator = $fkValidator;
        $this->braintreeConfig = $braintreeConfig;
        $this->tokenRepository = $tokenRepository;
        $this->paymentTokenManagement = $paymentTokenManagement;
        parent::__construct($context);

        $this->errorsMap = [
            self::WRONG_TOKEN => __('No token found.'),
            self::WRONG_REQUEST => __('Wrong request.'),
            self::ACTION_EXCEPTION => __('Card action failure. Please try again.')
        ];

        if ($customerSession->isLoggedIn()) {
            $this->initBraintreeSession($customerSession);
        }
    }

    protected function isValidRequest(): Int
    {
        $request = $this->_request;
        if (!$request instanceof Http) {
            return self::WRONG_REQUEST;
        }

        if (!$this->fkValidator->validate($request)) {
            return self::WRONG_REQUEST;
        }

        $this->request = $request;

        $payloadNonce = $this->request->getPostValue('cc_nonce');
        if (empty($payloadNonce)) {
            return self::WRONG_TOKEN;
        }
        $this->payloadNonce = $payloadNonce;

        return self::OK;
    }

    /**
     * @return String
     */
    protected function getPayloadNonce(): String
    {
        return $this->payloadNonce;
    }

    /**
     * @return Http
     */
    protected function getRequestInstance(): HTTP
    {
        return $this->request;
    }

    /**
     * @param Int $errorCode
     * @return ResponseInterface
     */
    protected function createErrorResponse(Int $errorCode): ResponseInterface
    {
        $this->messageManager->addErrorMessage(
            $this->errorsMap[$errorCode]
        );

        return $this->_redirect('vault/cards/listaction');
    }

    /**
     * @return ResponseInterface
     */
    protected function createSuccessMessage(): ResponseInterface
    {
        $this->messageManager->addSuccessMessage(
            __('Stored Payment Method was successfully saved')
        );
        return $this->_redirect('vault/cards/listaction');
    }

    /**
     * @param Session $customerSession
     */
    private function initBraintreeSession(Session $customerSession): void
    {
        $this->customer = $customerSession->getCustomer();
        $storeId = $this->customer->getStoreId();
        $this->braintreeGateway = new BraintreeGateway([
            'environment' => $this->braintreeConfig->getEnvironment($storeId),
            'merchantId' => $this->braintreeConfig->getMerchantId($storeId),
            'publicKey' => $this->braintreeConfig->getValue(BraintreeConfig::KEY_PUBLIC_KEY, $storeId),
            'privateKey' => $this->braintreeConfig->getValue(BraintreeConfig::KEY_PRIVATE_KEY, $storeId)
        ]);
    }
}
